using Libdl

function _find_gcclibdir()

  gcc_bin = Sys.which("gcc")

  if  "/usr/bin" == gcc_bin[1:8]
    return _find_gcclibdir_in_system()
  else
    return _find_gcclibdir_custom()
  end

end

function _find_gcclibdir_in_system()
  for v in [5,6,7,8,9]
    gcclibdir = "/usr/lib/gcc/x86_64-linux-gnu/$v"
    if isdir(gcclibdir)
      l = Libdl.find_library(joinpath(gcclibdir,"libgomp"))
      if l != ""
        return gcclibdir
      end
    end
  end
  s = """
  libgomp could not be found in system. Try
  \$ sudo apt-get install libgomp1
  """
  error(s)
  ""
end

function _find_gcclibdir_custom()
  gcc_bin = Sys.which("gcc")
  gcc_root = gcc_bin[1:(end-8)]
  gcclibdir = joinpath(gcc_root,"lib64")
  if ! isdir(gcclibdir)
    error("lib64 directory not gound in gcc instalation: $gcclibdir")
  end
  gcclibdir
end

deps_jl = "deps.jl"

if isfile(deps_jl)
  rm(deps_jl)
end

if !haskey(ENV,"MKLROOT")
  s = """
  Environment variable MKLROOT not found.
  
  Please install intel mkl math lbrary and rebuild.
  """
  error(s)
end

mklroot = ENV["MKLROOT"]

@info "MKLROOT found at: $mklroot"

mkllibdir = joinpath(mklroot,"lib/intel64")

if ! isdir(mkllibdir)
  s = """
  MKL lib directory not found: $mkllibdir
  """
  error(s)
end

@info "MKL libraries found at: $mkllibdir"

gcclibdir = _find_gcclibdir()

if ! isdir(gcclibdir)
  s = """
  gcc lib directory not found: $gcclibdir
  """
  error(s)
end

@info "gcc libraries found at: $gcclibdir"

open(deps_jl,"w") do f
  println(f, "# This file is automatically generated")
  println(f, "# Do not edit")
  println(f)
  println(f, :(const mklroot = $mklroot))
  println(f, :(const mkllibdir = $mkllibdir))
  println(f, :(const gcclibdir = $gcclibdir))
end

